package ejbs;
/*
 *  Copyright © - 2014 Anahata Technologies.
 */

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.EJBContext;
import javax.ejb.Stateless;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.inject.Inject;
import javax.jms.*;

/**
 *
 * @author Pablo Rodriguez Pina <pablo@anahata-it.com.au>
 */
@JMSConnectionFactoryDefinition(
        name = "java:module/jms/annotationCF",
        maxPoolSize = 30,
        minPoolSize = 20,
        interfaceName = "javax.jms.TopicXAConnectionFactory",
        properties = {
            "addressList=mq://localhost:7676",
            "reconnectEnabled=true"
        }
)
@JMSDestinationDefinition(
        name = "java:module/jms/annotationTopic",
        interfaceName = "javax.jms.Topic",
        description = "Yam Dms Event Topic",
        destinationName = "annotationTopic"
)
@Stateless
@TransactionManagement(TransactionManagementType.CONTAINER)
public class JmsBean {
    @Resource(lookup = "java:module/jms/annotationCF")
    private ConnectionFactory annotationCF;

    @Inject
    @JMSConnectionFactory("java:module/jms/annotationCF")
    @JMSSessionMode(JMSContext.SESSION_TRANSACTED)
    private JMSContext annotationInjectedContext;

    @Resource(lookup = "java:module/jms/annotationTopic")
    private Topic annotationTopic;

    @Resource(lookup = "jms/gf-resourcesCF")
    private ConnectionFactory resourcesCF;
//

    @Inject
    @JMSConnectionFactory("jms/gf-resourcesCF")
    private JMSContext resourcesInjectedContext;
//

    @Resource(lookup = "jms/resourcesTopic")
    private Topic resourcesTopic;

    @Resource
    private EJBContext ctx;

    @PostConstruct
    public void postConstruct() {
        System.out.println("annotationCF=" + annotationCF);
        System.out.println("resourcesCF=" + resourcesCF);
    }

    public void sendMessages() {
        sendMessage11(annotationCF, annotationTopic, "annotationCF + annotationTopic 1.1");
        sendMessage11(annotationCF, resourcesTopic, "annotationCF + resourcesTopic 1.1");
        sendMessage11(resourcesCF, resourcesTopic, "resourcesCF + annotationTopic 1.1");
        sendMessage11(resourcesCF, annotationTopic, "resourcesCF + annotationTopic 1.1");

        sendMessage20CreateContext(annotationCF, annotationTopic, "annotationCF + annotationTopic + created context 2.0");
        sendMessage20CreateContext(annotationCF, resourcesTopic, "annotationCF + resourcesTopic + created context 2.0");
        sendMessage20CreateContext(resourcesCF, resourcesTopic, "resourcesCF + resourcesTopic + created context 2.0");
        sendMessage20CreateContext(resourcesCF, annotationTopic, "resourcesCF + annotationTopic + created context 2.0");

        sendMessage20(annotationInjectedContext, annotationTopic, "annotationInjectedContext + annotationTopic 2.0");
        sendMessage20(annotationInjectedContext, resourcesTopic, "annotationInjectedContext + annotationTopic 2.0");
        sendMessage20(resourcesInjectedContext, resourcesTopic, "resourcesInjectedContext + resourcesTopic 2.0");
        sendMessage20(resourcesInjectedContext, annotationTopic, "resourcesInjectedContext + resourcesTopic 2.0");
        System.out.println("rolling back on sendMessages()");
        ctx.setRollbackOnly();
    }

    private void sendMessage20CreateContext(ConnectionFactory cf, Topic topic, String message) {
        try (JMSContext ctx = cf.createContext()) {
            sendMessage20(ctx, topic, message);
        }
    }

    private void sendMessage20(JMSContext context, Topic topic, String message) {
        JMSProducer producer = context.createProducer();
        producer.send(topic, message);
    }

    private void sendMessage11(ConnectionFactory cf, Topic topic, String message) {
        try {
            try (Connection con = cf.createConnection()) {
                try (Session s = con.createSession(true, 0)) {
                    MessageProducer mp = s.createProducer(topic);
                    TextMessage om = s.createTextMessage();
                    om.setText(message);
                    mp.send(om);
                }
            }

        } catch (JMSException e) {
            e.printStackTrace();
        }

    }
}
